#**************************************************************************
#   Copyright (C) 2009 by Michael Andreen <harvATruinDOTnu>               *
#                                                                         *
#   This program is free software; you can redistribute it and/or modify  *
#   it under the terms of the GNU General Public License as published by  *
#   the Free Software Foundation; either version 2 of the License, or     *
#   (at your option) any later version.                                   *
#                                                                         *
#   This program is distributed in the hope that it will be useful,       *
#   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
#   GNU General Public License for more details.                          *
#                                                                         *
#   You should have received a copy of the GNU General Public License     *
#   along with this program; if not, write to the                         *
#   Free Software Foundation, Inc.,                                       *
#   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.         *
#**************************************************************************/
package NDIRC::Delling;

use strict;
use warnings;
use feature ':5.10';

use Moose;
extends 'NDIRC::Bot';

use POE::Session;
use ND::DB;

use AI::MegaHAL;

has hal => (
	is => 'ro',
	isa => 'Object',
	lazy_build => 1,
);

after irc_public => sub {
	my ($self,$sender, $who, $where, $msg) = @_[OBJECT,SENDER, ARG0 .. ARG2];
	my ($nick,$username,$address) = ( split /[!@]/, $who );
	my $channel = $where->[0];

	my $irc = $sender->get_heap();

	#$irc->yield(privmsg => $channel, "Authed? " . $irc->is_nick_authed($nick));
	my $dbh = DB();
	my $seen = $dbh->prepare_cached(q{UPDATE users SET laston = NOW() WHERE hostmask = ?});
	$seen->execute($address);

	my $nickname = $irc->nick_name;
	if ($msg =~ /^$nickname: (.*)/i){
		my $text = $self->hal->do_reply($1);
		$irc->yield(privmsg => $channel, "$nick: $text");
	}elsif ($msg =~ /^(\S+): (.+)$/ && $self->disp->has_command('anon',$channel)){
		my $_ = $1;
		my $text = $2;
		unless ($irc->is_channel_member($channel,$1) || /(Constructing|Researching)/){
			$msg = ".anon $_ $text";
		}

	}
	if ($self->parseCommand($msg,$irc,$nick,$address,$channel,$dbh)){
		#Command parsed and run successfully
	}
};

after irc_msg => sub {
	my ($self,$sender, $who, $where, $msg) = @_[OBJECT,SENDER, ARG0 .. ARG2];
	my ($nick,$username,$address) = ( split /[!@]/, $who );
	my $irc = $sender->get_heap();

	my $dbh = DB();
	my $seen = $dbh->prepare_cached(q{UPDATE users SET laston = NOW() WHERE hostmask = ?});
	$seen->execute($address);

	if ($self->parseCommand($msg,$irc,$nick,$address,'pm',$dbh)){
		#Command parsed and run successfully
	}else{
		$irc->yield(notice => $nick, "unknown command");
	}
};

sub irc_join {
	my ($self,$sender, $who, $channel) = @_[OBJECT,SENDER, ARG0 .. ARG1];
	my ($nick,$username,$address) = ( split /[!@]/, $who );
	my $irc = $sender->get_heap();

	my $dbh = DB();
	my $seen = $dbh->prepare_cached(q{UPDATE users SET laston = NOW() WHERE hostmask = ?});
	$seen->execute($address);

	if($self->disp->has_command('voice',$channel)){
		my $flags = $dbh->prepare_cached(q{
SELECT DISTINCT flag
FROM users u
	JOIN groupmembers g USING (uid)
	JOIN channel_group_flags gf USING (gid)
WHERE u.hostmask = $1 AND channel = $2 AND flag IN ('o','v');
		});
		$flags->execute($address, $channel);
		my $mode = '';
		my @who;
		while (my ($flag) = $flags->fetchrow()){
			$mode .= $flag;
			push @who, $nick;
		}
		say "$mode - @who";
		$irc->yield(mode => $channel, $mode, @who) if $mode;
	}
}

sub refresh {
	my ($kernel,$heap) = @_[KERNEL,HEAP];
	$kernel->delay( refresh => 60 );
	print 'Time: ' . time() . ' Lag: ' . $heap->{connector}->lag() . "\n";

	my $dbh = DB();
	my $scans = $dbh->prepare(q{SELECT s.scan_id
			,coords(x,y,z),type
			,array_agg(sr.nick) AS nick
			,array_agg(sr.id) AS id
		FROM scan_requests sr
			JOIN scans s USING (pid,type)
			JOIN current_planet_stats USING (pid)
		WHERE sr.time > NOW() - '30 min'::INTERVAL
			AND s.tick >= sr.tick AND NOT sr.sent
		GROUP BY scan_id,x,y,z,type
		});
	my $sentscan = $dbh->prepare(q{UPDATE scan_requests
		SET sent = TRUE WHERE id = ANY($1)
		});
	$scans->execute;
	while (my $scan = $scans->fetchrow_hashref){
		$heap->{irc}->yield(notice => $scan->{nick}, "($scan->{coords} $scan->{type})"
			." http://game.planetarion.com/showscan.pl?scan_id=$scan->{scan_id}");
		$sentscan->execute($scan->{id});
	}
	return;
}

sub _build_hal {
	my $hal = AI::MegaHAL->new('Path' => '.', 'Banner' => 0, 'Prompt' => 0, 'Wrap' => 0, 'AutoSave' => 1);
	return $hal;
}

after sig_usr2 => sub {
	my $self = shift;
	say 'Saving brain!';
	$self->hal->_cleanup;
};

1;
